@@ -622,12 +622,7 @@ var LibraryGLFW = {
     },
 
     exitFullscreen: function() {
-      var CFS = document['exitFullscreen'] ||
-                document['cancelFullScreen'] ||
-                document['mozCancelFullScreen'] ||
-                document['webkitCancelFullScreen'] ||
-          (function() {});
-      CFS.apply(document, []);
+      Browser.exitFullscreen();
     },
 
     cancelFullScreen: function() {
@@ -758,6 +753,7 @@ var LibraryGLFW = {
 
       event.preventDefault();
 
+#if '$FS' in addedLibraryItems
       var filenames = allocate(new Array(event.dataTransfer.files.length*4), 'i8*', ALLOC_NORMAL);
       var filenamesArray = [];
       var count = event.dataTransfer.files.length;
@@ -799,6 +795,8 @@ var LibraryGLFW = {
         save(event.dataTransfer.files[i]);
       }
 
+#endif // '$FS' in addedLibraryItems
+
       return false;
     },
 
